from .elements import Empty, Field, Fieldset, List, Section
from .error_types import Error, ParseError, ValidationError
from .loaders import bootstrap_loaders
from .parse import parse

Fieldset.string = Fieldset.entry
List.string_items = List.items
Section.string = Section.field
Section.string_list = Section.list
Field.string = Field.value

def bootstrapped_entry_loader_factory(func):
  def bootstrapped_loader(self, name, **kwargs):
    return self.entry(name, func, **kwargs)

  return bootstrapped_loader

def bootstrapped_field_loader_factory(func):
  def bootstrapped_loader(self, name, **kwargs):
    return self.field(name, func, **kwargs)

  return bootstrapped_loader

def bootstrapped_items_loader_factory(func):
  def bootstrapped_loader(self, **kwargs):
    return self.items(func, **kwargs)

  return bootstrapped_loader

def bootstrapped_list_loader_factory(func):
  def bootstrapped_loader(self, name, **kwargs):
    return self.list(name, func, **kwargs)

  return bootstrapped_loader

def bootstrapped_value_loader_factory(func):
  def bootstrapped_loader(self, **kwargs):
    return self.value(func, **kwargs)

  return bootstrapped_loader

for name, func in bootstrap_loaders.items():
  setattr(Field, name, bootstrapped_value_loader_factory(func))
  setattr(Fieldset, name, bootstrapped_entry_loader_factory(func))
  setattr(List, f"{name}_items", bootstrapped_items_loader_factory(func))
  setattr(Section, name, bootstrapped_field_loader_factory(func))
  setattr(Section, f"{name}_list", bootstrapped_list_loader_factory(func))
