from collections import deque
from .errors import Analysis as errors


def analyze(context):
  context['document_instruction'] = {
    'depth': 0,
    'index': 0,
    'length': 0,
    'line': 1,
    'name': '<>#:=|\\_ENO_DOCUMENT',
    'ranges': {
      'section_operator': [0, 0],
      'name': [0, 0]
    },
    'subinstructions': deque()
  }

  context['unresolved_instructions'] = []
  context['template_index'] = {}

  last_continuable_instruction = None
  last_fieldset_entry_names = None
  last_name_instruction = None
  last_section_instruction = context['document_instruction']
  active_section_instructions = [context['document_instruction']]
  unresolved_idle_instructions = []

  for instruction in context['instructions']:
    if instruction['type'] == 'EMPTY_LINE' or instruction['type'] == 'COMMENT':
      if last_name_instruction:
        unresolved_idle_instructions.append(instruction)
      else:
        last_section_instruction['subinstructions'].append(instruction)

      continue

    if instruction['type'] == 'BLOCK_CONTENT':
      last_name_instruction['subinstructions'].append(instruction)
      continue

    if instruction['type'] == 'FIELD':
      last_section_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      instruction['subinstructions'] = deque()

      last_continuable_instruction = instruction
      last_name_instruction = instruction

      if instruction['name'] in context['template_index']:
        context['template_index'][instruction['name']].append(instruction)
      else:
        context['template_index'][instruction['name']] = [instruction]

      last_section_instruction['subinstructions'].append(instruction)

      continue

    if instruction['type'] == 'NAME':
      last_section_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      instruction['subinstructions'] = deque()

      last_continuable_instruction = instruction
      last_name_instruction = instruction

      if 'template' in instruction:
        context['unresolved_instructions'].append(instruction)

      if 'template' not in instruction or instruction['name'] != instruction['template']:
        if instruction['name'] in context['template_index']:
          context['template_index'][instruction['name']].append(instruction)
        else:
          context['template_index'][instruction['name']] = [instruction]

      last_section_instruction['subinstructions'].append(instruction)

      continue

    if instruction['type'] == 'LIST_ITEM':
      if not last_name_instruction:
        raise errors.missing_name_for_list_item(context, instruction)

      last_name_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      instruction['name'] = last_name_instruction['name'] # TODO: List items should just stay unnamed actually?!
      instruction['subinstructions'] = deque()

      if last_name_instruction['type'] == 'LIST':
        last_name_instruction['subinstructions'].append(instruction)
        last_continuable_instruction = instruction
        continue

      if last_name_instruction['type'] == 'NAME':
        last_name_instruction['type'] = 'LIST'
        last_name_instruction['subinstructions'].append(instruction)
        last_continuable_instruction = instruction
        continue

      if last_name_instruction['type'] == 'FIELDSET':
        raise errors.list_item_in_fieldset(context, instruction, last_name_instruction)

      if last_name_instruction['type'] == 'FIELD':
        raise errors.list_item_in_field(context, instruction, last_name_instruction)

    if instruction['type'] == 'FIELDSET_ENTRY':
      if not last_name_instruction:
        raise errors.missing_name_for_fieldset_entry(context, instruction)

      last_name_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      instruction['subinstructions'] = deque()

      if last_name_instruction['type'] == 'FIELDSET':
        if instruction['name'] in last_fieldset_entry_names:
          raise errors.duplicate_fieldset_entry_name(context, last_name_instruction, instruction)

        last_fieldset_entry_names.append(instruction['name'])
        last_name_instruction['subinstructions'].append(instruction)
        last_continuable_instruction = instruction
        continue

      if last_name_instruction['type'] == 'NAME':
        last_name_instruction['type'] = 'FIELDSET'
        last_name_instruction['subinstructions'].append(instruction)
        last_fieldset_entry_names = [instruction['name']]
        last_continuable_instruction = instruction
        continue

      if last_name_instruction['type'] == 'LIST':
        raise errors.fieldset_entry_in_list(context, instruction, last_name_instruction)

      if last_name_instruction['type'] == 'FIELD':
        raise errors.fieldset_entry_in_field(context, instruction, last_name_instruction)

    if instruction['type'] == 'BLOCK':
      last_section_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      instruction['subinstructions'] = deque()

      last_continuable_instruction = None
      last_name_instruction = instruction

      if instruction['name'] in context['template_index']:
        context['template_index'][instruction['name']].append(instruction)
      else:
        context['template_index'][instruction['name']] = [instruction]

      last_section_instruction['subinstructions'].append(instruction)

      continue

    if instruction['type'] == 'BLOCK_TERMINATOR':
      last_name_instruction['subinstructions'].append(instruction)
      last_name_instruction = None
      continue

    if instruction['type'] == 'CONTINUATION':
      if not last_continuable_instruction:
        raise errors.missing_element_for_continuation(context, instruction)

      last_continuable_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      if last_name_instruction['type'] == 'NAME':
        last_name_instruction['type'] = 'FIELD'

      last_continuable_instruction['subinstructions'].append(instruction)
      continue

    if instruction['type'] == 'SECTION':
      last_section_instruction['subinstructions'].extend(unresolved_idle_instructions)
      unresolved_idle_instructions = []

      if instruction['depth'] - last_section_instruction['depth'] > 1:
        raise errors.section_hierarchy_layer_skip(context, instruction, last_section_instruction)

      if instruction['depth'] > last_section_instruction['depth']:
        last_section_instruction['subinstructions'].append(instruction)
      else:
        while active_section_instructions[-1]['depth'] >= instruction['depth']:
          active_section_instructions.pop()

        active_section_instructions[-1]['subinstructions'].append(instruction)

      last_continuable_instruction = None
      last_name_instruction = None
      last_section_instruction = instruction
      active_section_instructions.append(instruction)

      if 'template' in instruction:
        context['unresolved_instructions'].append(instruction)

      if 'template' not in instruction or instruction['name'] != instruction['template']:
        if instruction['name'] in context['template_index']:
          context['template_index'][instruction['name']].append(instruction)
        else:
          context['template_index'][instruction['name']] = [instruction]

      instruction['subinstructions'] = deque()

      continue
  # ... ends for instruction in context['instructions']:

  if len(unresolved_idle_instructions) > 0:
    last_section_instruction['subinstructions'].extend(unresolved_idle_instructions)
    unresolved_idle_instructions = []
