from . import Field
from ..errors import Validation as errors


class Fieldset:
  def __init__(self, context, instruction, parent, from_empty=False):
    self.context = context
    self.instruction = instruction
    self.name = instruction['name']
    self.parent = parent

    self._entries = []
    self.entries_associative = {}

    if from_empty:
      self._enforce_all_elements = False
      self.touched = True
    else:
      instruction['element'] = self

      self._enforce_all_elements = parent._enforce_all_elements
      self.touched = False

      for subinstruction in instruction['subinstructions']:
        if subinstruction['type'] == 'FIELDSET_ENTRY':
          entry = Field(context, subinstruction, self)

          self._entries.append(entry)
          self.entries_associative[subinstruction['name']] = entry

          subinstruction['element'] = entry
        else:
          subinstruction['element'] = self

  def __repr__(self):
    return f"<class Fieldset name=\"{self.name}\" entries={len(self._entries)}>"

  def assert_all_touched(self, *, only=None, skip=None):
    for entry in self._entries:
      if (skip and entry.name in skip) or (only and entry.name not in only):
        continue

      if not entry.touched:
        raise errors.excess_name(self.context, entry['instruction'])

  def element(self, name, *, enforce_element=True, required=None):
    if required is not None:
      enforce_element = required

    self.touched = True

    if name not in self.entries_associative:
      if enforce_element:
        raise errors.missing_fieldset_entry(self.context, name, self.instruction)

      return None

    return self.entries_associative[name]

  def elements(self):
    self.touched = True
    return self._entries

  def enforce_all_elements(self, enforce=True):
    self._enforce_all_elements = enforce

  def entries(self):
    return self.elements()

  def entry(self,
            name,
            loader=None,
            *,
            enforce_element=None,
            enforce_value=False,
            required=None,
            with_element=False):

    if required is not None:
      enforce_value = required
    if enforce_element is None:
      enforce_element = self._enforce_all_elements
    enforce_element = enforce_element or enforce_value

    self.touched = True

    if name not in self.entries_associative:
      if enforce_element:
        raise errors.missing_fieldset_entry(self.context, name, self.instruction)
      return [None, None] if with_element else None

    element = self.entries_associative[name]

    element.touch()

    if with_element:
      return [element, element.value(loader, enforce_value=enforce_value)]
    else:
      return element.value(loader, enforce_value=enforce_value)

  def raw(self):
    return {
      self.name: [{ entry.name: entry.value() } for entry in self._entries]
    }

  def touch(self, *, entries=False):
    self.touched = True
    if entries:
      for entry in self._entries:
        entry.touch()
