from . import Field
from ..errors import Validation as errors


class List:
  def __init__(self, context, instruction, parent, from_empty=False):
    self.context = context
    self.instruction = instruction
    self.name = instruction['name']
    self.parent = parent
    self.touched = False
    self._items = []

    if from_empty:
      return

    instruction['element'] = self

    for subinstruction in instruction['subinstructions']:
      if subinstruction['type'] == 'LIST_ITEM':
        subinstruction['element'] = Field(context, subinstruction, self)
        self._items.append(subinstruction['element'])
      else:
        subinstruction['element'] = self

  def __repr__(self):
    return f"<class List name=\"{self.name}\" items={len(self._items)}>"

  def elements(self):
    self.touched = True
    return self._items

  def items(self,
            loader=None,
            *,
            elements=False,
            enforce_values=True,
            with_elements=False):

    self.touched = True

    if elements:
      return self._items

    if with_elements:
      return [[item, item.value(loader, enforce_value=enforce_values)] for item in self._items]
    else:
      return [item.value(loader, enforce_value=enforce_values) for item in self._items]

  def length(self):
    return len(self._items)

  def raw(self):
    return { self.name: [item.value() for item in self._items] }

  def touch(self, *, items=False):
    self.touched = True
    if items:
      for item in self._items:
        item.touch()
