from ..error_types import ParseError


class Analysis:
  @staticmethod
  def fieldset_entry_in_field(context, entry_instruction, field_instruction):
    message = context['messages']['analysis']['fieldset_entry_in_field'](
      entry_instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(
      context,
      entry_instruction,
      [field_instruction, *field_instruction['subinstructions']]
    )

    selection = [
      [entry_instruction['line'], 0],
      [entry_instruction['line'], entry_instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def fieldset_entry_in_list(context, entry_instruction, list_instruction):
    message = context['messages']['analysis']['fieldset_entry_in_list'](
      entry_instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(
      context,
      entry_instruction,
      [list_instruction, *list_instruction['subinstructions']]
    )

    selection = [
      [entry_instruction['line'], 0],
      [entry_instruction['line'], entry_instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def duplicate_fieldset_entry_name(context, fieldset_instruction, entry_instruction):
    previous_entry_instruction = next(
      (i for i in fieldset_instruction['subinstructions'] if i['name'] == entry_instruction['name'])
    )

    message = context['messages']['analysis']['duplicate_fieldset_entry_name'](
      fieldset_instruction['name'],
      entry_instruction['name']
    )

    snippet = context['reporter'].report(
      context,
      [entry_instruction, previous_entry_instruction],
      [fieldset_instruction, *fieldset_instruction['subinstructions']]
    )

    selection = [
      [entry_instruction['line'], 0],
      [entry_instruction['line'], entry_instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def list_item_in_fieldset(context, item_instruction, fieldset_instruction):
    message = context['messages']['analysis']['list_item_in_fieldset'](
      item_instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(
      context,
      item_instruction,
      [fieldset_instruction, *fieldset_instruction['subinstructions']]
    )

    selection = [
      [item_instruction['line'], 0],
      [item_instruction['line'], item_instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def list_item_in_field(context, item_instruction, field_instruction):
    message = context['messages']['analysis']['list_item_in_field'](
      item_instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(
      context,
      item_instruction,
      [field_instruction, *field_instruction['subinstructions']]
    )

    selection = [
      [item_instruction['line'], 0],
      [item_instruction['line'], item_instruction['length']]
    ]

    return ParseError(message, snippet, selection)


  @staticmethod
  def missing_element_for_continuation(context, instruction):
    message = context['messages']['analysis']['missing_element_for_continuation'](
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, instruction)

    selection = [
      [instruction['line'], 0],
      [instruction['line'], instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def missing_name_for_fieldset_entry(context, instruction):
    message = context['messages']['analysis']['missing_name_for_fieldset_entry'](
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, instruction)

    selection = [
      [instruction['line'], 0],
      [instruction['line'], instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def missing_name_for_list_item(context, instruction):
    message = context['messages']['analysis']['missing_name_for_list_item'](
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, instruction)

    selection = [
      [instruction['line'], 0],
      [instruction['line'], instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def section_hierarchy_layer_skip(context, sub_section_instruction, super_section_instruction):
    message = context['messages']['analysis']['section_hierarchy_layer_skip'](
      sub_section_instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, [sub_section_instruction, super_section_instruction])

    selection = [
      [sub_section_instruction['line'], 0],
      [sub_section_instruction['line'], sub_section_instruction['length']]
    ]

    return ParseError(message, snippet, selection)
