import re
from ..error_types import ParseError


class Tokenization:
  UNTERMINATED_ESCAPED_NAME = re.compile('^\s*(`+)(?!`)((?:(?!\1).)+)$')

  @staticmethod
  def unterminated_escaped_name(context, instruction, selection_column):
    message = context['messages']['tokenization']['unterminated_escaped_name'](
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, instruction)

    selection = [
      [instruction['line'], selection_column],
      [instruction['line'], instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def invalid_line(context, instruction):
    line = context['input'][instruction['index']:instruction['index'] + instruction['length']]

    match = Tokenization.UNTERMINATED_ESCAPED_NAME.match(line)
    if match is not None:
      return Tokenization.unterminated_escaped_name(context, instruction, match.end(1))

    message = context['messages']['tokenization']['invalid_line'](
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(context, instruction)

    selection = [
      [instruction['line'], 0],
      [instruction['line'], instruction['length']]
    ]

    return ParseError(message, snippet, selection)

  @staticmethod
  def unterminated_block(context, instruction):
    block_content_instructions = [i for i in context['instructions'] if i['line'] > instruction['line']]

    message = context['messages']['tokenization']['unterminated_block'](
      instruction['name'],
      instruction['line'] + context['indexing']
    )

    snippet = context['reporter'].report(
      context,
      instruction,
      block_content_instructions
    )

    selection = [
      [instruction['line'], instruction['ranges']['block_operator'][0]],
      [instruction['line'], instruction['ranges']['name'][1]]
    ]

    return ParseError(message, snippet, selection)
