# Note: Study this file from the bottom up

import re


class Grammar:
  OPTIONAL = '([^\\n]+?)?'
  REQUIRED = '(\\S[^\\n]*?)'

  #
  EMPTY = '()'
  EMPTY_LINE_INDEX = 1

  # | Value
  NEWLINE_CONTINUATION = f"(\\|)[^\\S\\n]*{OPTIONAL}"
  NEWLINE_CONTINUATION_OPERATOR_INDEX = 2
  NEWLINE_CONTINUATION_VALUE_INDEX = 3

  # \ Value
  LINE_CONTINUATION = f"(\\\\)[^\\S\\n]*{OPTIONAL}"
  LINE_CONTINUATION_OPERATOR_INDEX = 4
  LINE_CONTINUATION_VALUE_INDEX = 5

  CONTINUATION = f"{NEWLINE_CONTINUATION}|{LINE_CONTINUATION}"

  # > Comment
  COMMENT = f"(>)[^\\S\\n]*{OPTIONAL}"
  COMMENT_OPERATOR_INDEX = 6
  COMMENT_TEXT_INDEX = 7

  # - Value
  LIST_ITEM = f"(-)(?!-)[^\\S\\n]*{OPTIONAL}"
  LIST_ITEM_OPERATOR_INDEX = 8
  LIST_ITEM_VALUE_INDEX = 9

  # -- Name
  BLOCK = f"(-{{2,}})[^\\S\\n]*{REQUIRED}"
  BLOCK_DASHES_INDEX = 10
  BLOCK_NAME_INDEX = 11

  # #
  SECTION_HASHES = '(#+)(?!#)'
  SECTION_HASHES_INDEX = 12

  # # Name
  SECTION_NAME_UNESCAPED = '(?!`)([^\\s<][^<\\n]*?)'
  SECTION_NAME_UNESCAPED_INDEX = 13

  # # `Name`
  SECTION_NAME_ESCAPE_BEGIN_OPERATOR_INDEX = 14
  SECTION_NAME_ESCAPED = f"(`+)[^\\S\\n]*(\\S[^\\n]*?)[^\\S\\n]*(\\{SECTION_NAME_ESCAPE_BEGIN_OPERATOR_INDEX})" # TODO: Should this exclude the backreference inside the quotes? (as in ((?:(?!\\1).)+) ) here and elsewhere (probably not because it's not greedy.?)
  SECTION_NAME_ESCAPED_INDEX = 15
  SECTION_NAME_ESCAPE_END_OPERATOR_INDEX = 16

  # # Name < Template
  # # `Escaped section name` < Template
  SECTION_NAME = f"(?:{SECTION_NAME_UNESCAPED}|{SECTION_NAME_ESCAPED})"
  SECTION_TEMPLATE = f"(?:(<(?!<)|<<)[^\\S\\n]*{REQUIRED})?"
  SECTION = f"{SECTION_HASHES}\\s*{SECTION_NAME}[^\\S\\n]*{SECTION_TEMPLATE}"
  SECTION_COPY_OPERATOR_INDEX = 17
  SECTION_TEMPLATE_INDEX = 18

  EARLY_DETERMINED = f"{CONTINUATION}|{COMMENT}|{LIST_ITEM}|{BLOCK}|{SECTION}"

  # Name
  NAME_UNESCAPED = '(?![>#\\-`\\\\|])([^\\s:=<][^:=<]*?)'
  NAME_UNESCAPED_INDEX = 19

  # `Name`
  NAME_ESCAPE_BEGIN_OPERATOR_INDEX = 20
  NAME_ESCAPED = f"(`+)[^\\S\\n]*(\\S[^\\n]*?)[^\\S\\n]*(\\{NAME_ESCAPE_BEGIN_OPERATOR_INDEX})"
  NAME_ESCAPED_INDEX = 21
  NAME_ESCAPE_END_OPERATOR_INDEX = 22

  NAME = f"(?:{NAME_UNESCAPED}|{NAME_ESCAPED})"

  # :
  # : Value
  FIELD_OR_NAME = f"(:)[^\\S\\n]*{OPTIONAL}"
  NAME_OPERATOR_INDEX = 23
  FIELD_VALUE_INDEX = 24

  # =
  # = Value
  FIELDSET_ENTRY = f"(=)[^\\S\\n]*{OPTIONAL}"
  FIELDSET_ENTRY_OPERATOR_INDEX = 25
  FIELDSET_ENTRY_VALUE_INDEX = 26

  # < Template
  # << Template
  COPY = f"(<(?!<)|<<)\\s*{REQUIRED}"
  COPY_OPERATOR_INDEX = 27
  TEMPLATE_INDEX = 28

  LATE_DETERMINED = f"{NAME}\\s*(?:{FIELD_OR_NAME}|{FIELDSET_ENTRY}|{COPY})"

  NOT_EMPTY = f"(?:{EARLY_DETERMINED}|{LATE_DETERMINED})"

  REGEX = re.compile(f"[^\\S\\n]*(?:{EMPTY}|{NOT_EMPTY})[^\\S\\n]*(?=\\n|$)")
