from .analyzer import analyze
from .elements import Section
from .messages import MESSAGES
from .reporters import Text
from .resolver import Resolver
from .tokenizer import Tokenizer


def parse(input, locale='en', reporter=Text, zero_indexing=False):
  if not isinstance(input, str):
    raise TypeError(
      f"The parser accepts only strings as input, input was: {input}"
    )

  if locale not in MESSAGES:
    raise ValueError(
      f"The requested locale '{locale}' is not available. Translation contributions are " +
      f"greatly appreciated, visit https://github.com/eno-lang/eno-locales/ if you wish to contribute."
    )

  context = {
    'indexing': 0 if zero_indexing else 1,
    'input': input,
    'messages': MESSAGES[locale],
    'locale': locale,
    'reporter': reporter
  }

  tokenizer = Tokenizer(context)
  tokenizer.tokenize()

  analyze(context)

  resolver = Resolver(context)
  resolver.resolve()

  context['document'] = Section(context, context['document_instruction'], None)

  return context['document']
