from collections import deque
from html import escape


class HTML:
  @staticmethod
  def line(gutter, content, classes=[]):
    result = ''

    result += f"<div class=\"eno-report-line {' '.join(classes)}\">"
    result +=   f"<div class=\"eno-report-gutter\">{gutter.rjust(10)}</div>"
    result +=   f"<div class=\"eno-report-content\">{escape(content)}</div>"
    result += '</div>'

    return result

  @staticmethod
  def report(context, *emphasized_marked):
    if len(emphasized_marked) >= 2:
      emphasized, marked = emphasized_marked
    else:
      emphasized = emphasized_marked[0]
      marked = []

    if isinstance(emphasized, deque):
      emphasized = list(emphasized)
    elif not isinstance(emphasized, list):
      emphasized = [emphasized]

    if isinstance(marked, deque):
      marked = list(marked)
    elif not isinstance(marked, list):
      marked = [marked]

    content_header = context['messages']['reporting']['content_header']
    gutter_header = context['messages']['reporting']['gutter_header']
    omission = HTML.line('...', '...')

    snippet = '<pre class="eno-report">'

    snippet = ''

    if 'source_label' in context:
      snippet += f"<div>{context['source_label']}</div>"

    snippet += HTML.line(gutter_header, content_header)

    in_omission = False

    for instruction in context['instructions']:
      emphasize = instruction in emphasized
      mark = instruction in marked

      show = False

      for marked_instruction in (emphasized + marked):
        if (instruction['line'] >= marked_instruction['line'] - 2 and
            instruction['line'] <= marked_instruction['line'] + 2):
          show = True
          break

      if show:
        classes = []

        if emphasize:
          classes.append('eno-report-line-emphasized')
        elif mark:
          classes.append('eno-report-line-marked')

        snippet += HTML.line(
          str(instruction['line'] + context['indexing']),
          context['input'][instruction['index']:instruction['index'] + instruction['length']],
          classes=classes
        )

        in_omission = False
      elif not in_omission:
        snippet += omission
        in_omission = True

    snippet += '</pre>'

    return snippet
