from collections import deque


class Terminal:
  @staticmethod
  def report(context, *emphasized_marked):
    if len(emphasized_marked) >= 2:
      emphasized, marked = emphasized_marked
    else:
      emphasized = emphasized_marked[0]
      marked = []

    if isinstance(emphasized, deque):
      emphasized = list(emphasized)
    elif not isinstance(emphasized, list):
      emphasized = [emphasized]

    if isinstance(marked, deque):
      marked = list(marked)
    elif not isinstance(marked, list):
      marked = [marked]

    content_header = context['messages']['reporting']['content_header']
    gutter_header = context['messages']['reporting']['gutter_header'].rjust(5)

    gutter_width = len(gutter_header) + 3
    columns_header = f"\x1b[1m  {gutter_header} | {content_header}\x1b[0m\n"
    omission = f"\x1b[1m{' ' * (gutter_width - 5)}...\x1b[0m\n"

    snippet = ''

    if 'source_label' in context:
      snippet += f"\x1b[1m{context['source_label']}\x1b[0m\n"

    snippet += columns_header

    in_omission = False

    for instruction in context['instructions']:
      emphasize = instruction in emphasized
      mark = instruction in marked

      show = False

      for marked_instruction in (emphasized + marked):
        if (instruction['line'] >= marked_instruction['line'] - 2 and
            instruction['line'] <= marked_instruction['line'] + 2):
          show = True
          break

      if show:
        content = context['input'][instruction['index']:instruction['index'] + instruction['length']]
        number = str(instruction['line'] + context['indexing'])

        if emphasize:
          snippet += f"\x1b[41m >{number.rjust(gutter_width - 3)} | {content}\x1b[0m\n"
        elif mark:
          snippet += f"\x1b[33m *{number.rjust(gutter_width - 3)} | {content}\x1b[0m\n"
        else:
          snippet += f"{number.rjust(gutter_width - 1)} | {content}\n"

        in_omission = False
      elif not in_omission:
        snippet += omission
        in_omission = True

    return snippet
